--- osmocon.c	Sun Mar 14 14:22:01 2010
+++ osmocon.c	Sun Mar 21 18:41:28 2010
@@ -401,6 +401,11 @@
 			printf("Dropping sample '%c'\n", buffer[i]);
 }
 
+// Dieter: to avoid problems with code download under Cygwin, a "fast read" mode is introduced
+
+static int read_fast = 1; // start with fast read
+static int code_running = 0; // indicate if code is running
+
 static int handle_read(void)
 {
 	int rc, nbytes, buf_left;
@@ -446,6 +451,7 @@
 		dnload.state = WAITING_PROMPT1;
 		dnload.write_ptr = dnload.data;
 		dnload.print_hdlc = 1;
+		code_running = 1; // Dieter
 	} else if (!memcmp(buffer, phone_nack, sizeof(phone_nack))) {
 		printf("Received DOWNLOAD NACK from phone, something went wrong :(\n");
 		dnload.serial_fd.when = BSC_FD_READ;
@@ -462,6 +468,13 @@
 		dnload.state = WAITING_PROMPT1;
 		dnload.write_ptr = dnload.data;
 	}
+#if 1 // Dieter: switch to "fast read" mode in advance before complete PROMPT1 has been received
+	#define CHECK_ADVANCE 4
+	else if (!memcmp(buffer + CHECK_ADVANCE, phone_prompt1, sizeof(phone_prompt1) - CHECK_ADVANCE)) {
+		read_fast = 1;
+	}
+#endif
+
 	bufptr += nbytes;
 
 	return nbytes;
@@ -471,6 +484,8 @@
 {
 	int rc;
 
+#if 0 // original code
+
 	if (flags & BSC_FD_READ) {
 		rc = handle_read();
 		if (rc == 0)
@@ -482,6 +497,94 @@
 		if (rc == 1)
 			dnload.state = WAITING_PROMPT1;
 	}
+
+#else // Dieter: to avoid problems with timing-critical code download on Cygwin
+
+	if (flags & BSC_FD_READ) {
+		rc = handle_read();
+		if (rc == 0)
+			exit(2);
+
+		if(read_fast)
+		{
+			// turn off non-blocking mode
+
+			int flags = fcntl(dnload.serial_fd.fd, F_GETFL);
+			if(flags < 0)
+				printf("Error 1: %d\n", flags);
+			flags &= ~O_NONBLOCK;
+			fcntl(dnload.serial_fd.fd, F_SETFL, flags);
+			if(flags < 0)
+				printf("Error 2: %d\n", flags);
+
+			// handle code download to bootloader as fast as possible
+			// without leaving the loop before done.
+
+			for(;;) {
+				if(dnload.state != DOWNLOADING) {	
+					code_running = 0;
+					rc = handle_read();
+					if (rc == 0)
+						exit(2);
+					if(code_running == 1) // code is running, exit
+						break;
+				}
+
+				if(dnload.state == DOWNLOADING) {
+					rc = handle_write();
+					if (rc == 1) {
+						dnload.state = WAITING_PROMPT1;
+						break;
+					}
+				}
+			} 
+
+			read_fast = 0;
+			dnload.serial_fd.when = BSC_FD_READ; // otherwise code download will not end
+
+			// turn on non-blocking mode again
+
+			flags = fcntl(dnload.serial_fd.fd, F_GETFL);
+			if(flags < 0)
+				printf("Error 3: %d\n", flags);
+			flags |= O_NONBLOCK;
+			fcntl(dnload.serial_fd.fd, F_SETFL, flags);
+			if(flags < 0)
+				printf("Error 4: %d\n", flags);
+
+			return;
+		}
+	}
+
+	if (flags & BSC_FD_WRITE) {
+		rc = handle_write();
+		if (rc == 1)
+			dnload.state = WAITING_PROMPT1;
+	}
+#endif
+
 	return 0;
 }
 
